package at.tugraz.iaik.p2p.kademlia.dht;

import at.tugraz.iaik.p2p.kademlia.node.Identifier;

/**
 * A GET request can get content based on Key, Owner, Type, etc
 * <p>
 * This is a class containing the parameters to be passed in a GET request
 * <p>
 * We use a class since the number of filtering parameters can change later
 *
 * @author Joshua Kissoon
 * @since 20140224
 */
public class GetParameter {

    private final Identifier key;
    private String ownerId = null;
    private String type = null;

    /**
     * Construct a GetParameter to search for data by NodeId and owner
     *
     * @param key
     * @param type
     */
    public GetParameter(final Identifier key, final String type) {
        this.key = key;
        this.type = type;
    }

    /**
     * Construct a GetParameter to search for data by NodeId, owner, type
     *
     * @param key
     * @param type
     * @param owner
     */
    public GetParameter(final Identifier key, final String type, final String owner) {
        this(key, type);
        this.ownerId = owner;
    }

    /**
     * Construct our get parameter from a Content
     *
     * @param c
     */
    public GetParameter(final KadContent c) {
        this.key = c.getKey();

        if (c.getType() != null) {
            this.type = c.getType();
        }

        if (c.getOwnerId() != null) {
            this.ownerId = c.getOwnerId();
        }
    }

    /**
     * Construct our get parameter from a StorageEntryMeta data
     *
     * @param md
     */
    public GetParameter(final KademliaStorageEntryMetadata md) {
        this.key = md.getKey();

        if (md.getType() != null) {
            this.type = md.getType();
        }

        if (md.getOwnerId() != null) {
            this.ownerId = md.getOwnerId();
        }
    }

    public Identifier getKey() {
        return this.key;
    }

    public void setOwnerId(final String ownerId) {
        this.ownerId = ownerId;
    }

    public String getOwnerId() {
        return this.ownerId;
    }

    public void setType(final String type) {
        this.type = type;
    }

    public String getType() {
        return this.type;
    }

    @Override
    public String toString() {
        return "GetParameter - [Key: " + this.key + "][Owner: " + this.ownerId + "][Type: " + this.type + "]";
    }
}
