package at.tugraz.iaik.p2p.kademlia.message;

import at.tugraz.iaik.p2p.kademlia.node.Identifier;
import at.tugraz.iaik.p2p.kademlia.node.Node;
import at.tugraz.iaik.p2p.kademlia.node.SHA3ID;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
 * A message sent to other nodes requesting the K-Closest nodes to a key sent in this message.
 *
 * @author Joshua Kissoon
 * @created 20140218
 */
public class DataLookupMessage extends Message {


    private final SHA3ID lookupId;


    /**
     * A new NodeLookupMessage to find nodes
     *
     * @param origin The Node from which the message is coming from
     * @param lookup The key for which to lookup nodes for
     */
    public DataLookupMessage(final Node origin, Node target, final SHA3ID lookup, int commID) {
        super(origin, target, MsgCode.DLOOKUP, commID);
        this.lookupId = lookup;
    }

    DataLookupMessage(final DataInputStream in) throws IOException {
        super(MsgCode.DLOOKUP, in);
        lookupId = (SHA3ID) Identifier.fromStr(in);
    }


    @Override
    public void toStream(final DataOutputStream out) throws IOException {
        super.toStream(out);
        this.lookupId.toStream(out);
    }

    public SHA3ID getLookupId() {
        return this.lookupId;
    }

    @Override
    public String toString() {
        return super.toString() + ", DataLookupMessage ] lookup=" + this.lookupId;
    }
}
