package at.tugraz.iaik.p2p.kademlia.message;

import at.tugraz.iaik.p2p.kademlia.node.Node;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Calendar;

public abstract class Message implements OutStreamable {

    protected enum MsgCode {

        SIMPLE(0x00), NLOOKUP(0x01), NREPLY(0x02), PING(0x03), PONG(0x04), DLOOKUP(0x05), DREPLY(0x06);
        public final byte code;

        private MsgCode(int code) {
            this.code = (byte) code;
        }

        public static MsgCode fromByte(byte b) {
            for (MsgCode c : MsgCode.values()) {
                if (c.code == b) {
                    return c;
                }
            }
            return null;
        }
    }


    protected final Node origin, target;
    protected final byte code;
    protected final int  commID;
    protected final long timestamp;

    public Message(Node origin, Node target, MsgCode code, int commID) {
        this(code.code, commID, Calendar.getInstance().getTimeInMillis(), origin, target);
    }

    private Message(byte code, int commID, long timestamp, Node origin, Node target) {
        this.code = code;
        this.origin = origin;
        this.target = target;
        this.commID = commID;
        this.timestamp = timestamp;
    }

    protected Message(MsgCode code, DataInputStream in) throws IOException {
        this(code.code, in.readInt(), in.readLong(), new Node(in), new Node(in));
    }

    public void toStream(DataOutputStream out) throws IOException {
        out.writeByte(code);
        out.writeInt(commID);
        out.writeLong(timestamp);
        origin.toStream(out);
        target.toStream(out);
    }


    public Node getTarget() {
        return target;
    }

    public Node getOrigin() {
        return origin;
    }

    public long getTimestamp() {
        return timestamp;
    }

    public int getCommID() {
        return commID;
    }

    /**
     * The unique code for the message type, used to differentiate all messages
     * from each other. Since this is of <code>byte</code> type there can
     * be at most 256 different message types.
     *
     * @return byte A unique code representing the message type
     */
    public byte code() {
        return code;
    }

    public String toString() {
        return "[" + code() + ", " + String.format("%13d", getCommID()) + ", origin=" + origin;
    }

}
