package at.tugraz.iaik.p2p.kademlia.message;

import at.tugraz.iaik.p2p.kademlia.KadConfiguration;
import at.tugraz.iaik.p2p.kademlia.KadServer;
import at.tugraz.iaik.p2p.kademlia.KademliaNode;
import at.tugraz.iaik.p2p.kademlia.dht.KademliaDHT;

import java.io.DataInputStream;
import java.io.IOException;

/**
 * Handles creating messages and receivers
 *
 * @author Joshua Kissoon
 * @since 20140202
 */
public class MessageFactory {

    private final KademliaNode     localNode;
    private final KademliaDHT      dht;
    private final KadConfiguration config;

    public MessageFactory(final KademliaNode local, final KademliaDHT dht, final KadConfiguration config) {
        this.localNode = local;
        this.dht = dht;
        this.config = config;
    }

    public static Message createMessage(final DataInputStream in) throws IOException {
        Message.MsgCode code = Message.MsgCode.fromByte(in.readByte());
        switch (code) {
            case NLOOKUP:
                return new NodeLookupMessage(in);
            case NREPLY:
                return new NodeReplyMessage(in);
            case PING:
                return new PingMessage(in);
            case PONG:
                return new PongMessage(in);
            case SIMPLE:
                return new SimpleMessage(in);
            case DLOOKUP:
                return new DataLookupMessage(in);
            case DREPLY:
                return new DataReplyMessage(in);
            default:
                //System.out.println(this.localNode + " - No Message handler found for message. Code: " + code);
                return new SimpleMessage(in);
        }
    }

    public Receiver createReceiver(final byte codeByte, final KadServer server) {
        Message.MsgCode code = Message.MsgCode.fromByte(codeByte);
        switch (code) {
            case SIMPLE:
                return new SimpleReceiver();
            case PING:
                return new SimpleReceiver();
            case PONG:
                return new SimpleReceiver();
            case NLOOKUP:
                return new NodeLookupReceiver(server, this.localNode, this.config);
            case NREPLY:
                return new NodeReplyReceiver(server, localNode, config);
            case DLOOKUP:
                return new DataLookupReceiver(server, localNode, config);
            case DREPLY:
                return new DataReplyReceiver(server, localNode, config);
            default:
                System.out.println("No receiver found for message. Code: " + code);
                return new SimpleReceiver();
        }
    }
}
