package at.tugraz.iaik.p2p.kademlia.message;

import at.tugraz.iaik.p2p.kademlia.node.Identifier;
import at.tugraz.iaik.p2p.kademlia.node.Node;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
 * A message sent to other nodes requesting the K-Closest nodes to a key sent in this message.
 *
 * @author Joshua Kissoon
 * @created 20140218
 */
public class NodeLookupMessage extends Message {


    private final Identifier lookupId;


    /**
     * A new NodeLookupMessage to find nodes
     *
     * @param origin The Node from which the message is coming from
     * @param lookup The key for which to lookup nodes for
     */
    public NodeLookupMessage(final Node origin, Node target, final Identifier lookup, int commID) {
        super(origin, target, MsgCode.NLOOKUP, commID);
        this.lookupId = lookup;
    }

    NodeLookupMessage(final DataInputStream in) throws IOException {
        super(MsgCode.NLOOKUP, in);
        lookupId = Identifier.fromStr(in);
    }


    @Override
    public void toStream(final DataOutputStream out) throws IOException {
        super.toStream(out);
        this.lookupId.toStream(out);
    }

    public Identifier getLookupId() {
        return this.lookupId;
    }

    @Override
    public String toString() {
        return super.toString() + ", NodeLookupMessage ] lookup=" + this.lookupId;
    }
}
