package at.tugraz.iaik.p2p.kademlia.message;

import at.tugraz.iaik.p2p.kademlia.node.Node;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * A message used to connect nodes.
 * When a NodeLookup Request comes in, we respond with a NodeReplyMessage.
 *
 * @author Joshua Kissoon
 * @created 20140218
 */
public class NodeReplyMessage extends Message {


    private final List<Message> nodes;
    private final SimpleMessage originMessage;

    public NodeReplyMessage(final Node origin, Node target, int commID, final List<Message> nodes) {
        super(origin, target, MsgCode.NREPLY, commID);
        this.nodes = nodes;
        originMessage = new SimpleMessage(origin, origin, 0, "");

    }

    NodeReplyMessage(final DataInputStream in) throws IOException {
        super(MsgCode.NREPLY, in);
        try {

            this.originMessage = (SimpleMessage) MessageFactory.createMessage(in);
        } catch (Exception e) {
            throw new IOException("Protocol Violation!");
        }
        /* Get the number of incoming nodes */
        final int len = in.readInt();
        this.nodes = new ArrayList<>(len);

        /* Read in all nodes */
        for (int i = 0; i < len; i++) {
            this.nodes.add(MessageFactory.createMessage(in));
        }
    }


    @Override
    public void toStream(final DataOutputStream out) throws IOException {
        super.toStream(out);
        /* Add all other nodes to the stream */
        final int len = this.nodes.size();
        if (len > 255) {
            throw new IndexOutOfBoundsException("Too many nodes in list to send in NodeReplyMessage. Size: " + len);
        }

        /* Writing the nodes to the stream */
        originMessage.toStream(out);
        out.writeInt(len);
        for (final Message n : this.nodes) {
            n.toStream(out);
        }
    }

    public Node getOrigin() {
        return this.origin;
    }

    public SimpleMessage getOriginMessage() {
        return originMessage;

    }

    public List<Message> getNodes() {
        return this.nodes;
    }

    @Override
    public String toString() {
        return super.toString() +
               ", NodeReplyMessage] originMessage: " +
               originMessage +
               ", " +
               Arrays.toString(nodes.toArray());

    }
}
