package at.tugraz.iaik.p2p.kademlia.message;

import at.tugraz.iaik.p2p.kademlia.KadConfiguration;
import at.tugraz.iaik.p2p.kademlia.KadServer;
import at.tugraz.iaik.p2p.kademlia.KademliaNode;
import at.tugraz.iaik.p2p.kademlia.node.Node;

import java.io.IOException;
import java.util.List;

/**
 * Receives a NodeLookupMessage and sends a NodeReplyMessage as reply with the K-Closest nodes to the ID sent.
 *
 * @author Joshua Kissoon
 * @created 20140219
 */
public class NodeReplyReceiver implements Receiver {

    private final KadServer server;
    private final KademliaNode localNode;
    private final KadConfiguration config;

    public NodeReplyReceiver(final KadServer server, final KademliaNode local, final KadConfiguration config) {
        this.server = server;
        this.localNode = local;
        this.config = config;
    }

    /**
     * Handle receiving a NodeLookupMessage
     * Find the set of K nodes closest to the lookup ID and return them
     *
     * @param ignored
     * @throws IOException
     */
    @Override
    public void receive(final Message incoming, final int ignored) throws IOException {

        final NodeReplyMessage msg = (NodeReplyMessage) incoming;

        final Node origin = msg.getOrigin();

        /* Update the local space by inserting the origin node. */
        this.localNode.getRoutingTable().insert(msg.getOriginMessage());

        /* Find nodes closest to the LookupId */


        final List<Message> nodes = msg.getNodes();
        for (Message n : nodes)
            localNode.getRoutingTable().insert(n);
    }

    /**
     * We don't need to do anything here
     *
     * @param comm
     * @throws IOException
     */
    @Override
    public void timeout(final int comm) throws IOException {
    }
}
