package at.tugraz.iaik.p2p.kademlia.node;

import at.tugraz.iaik.p2p.kademlia.message.Message;

import java.math.BigInteger;
import java.util.Comparator;

/**
 * A Comparator to compare 2 keys to a given key
 *
 * @author Joshua Kissoon
 * @since 20140322
 */
public class KeyComparator implements Comparator<Message> {

    private final BigInteger key;

    /**
     * @param key The NodeId relative to which the distance should be measured.
     */
    public KeyComparator(final Identifier key) {
        this.key = key.getInt();
    }

    /**
     * Compare two objects which must both be of type <code>Node</code>
     * and determine which is closest to the identifier specified in the
     * constructor.
     *
     * @param n1 Node 1 to compare distance from the key
     * @param n2 Node 2 to compare distance from the key
     */
    @Override
    public int compare(final Message n1, final Message n2) {
        BigInteger b1 = n1.getOrigin().getNodeId().getInt();
        BigInteger b2 = n2.getOrigin().getNodeId().getInt();
        return compare(b1, b2, key);
    }

    public static int compare(byte[] a, byte[] b, BigInteger key) {
        BigInteger b1 = new BigInteger(1, a);
        BigInteger b2 = new BigInteger(1, b);
        return compare(b1, b2, key);
    }

    public static int compare(BigInteger b1, BigInteger b2, BigInteger key) {
        b1 = b1.xor(key);
        b2 = b2.xor(key);
        return b1.abs().compareTo(b2.abs());
    }

}
