package at.tugraz.iaik.p2p.kademlia.node;

import at.tugraz.iaik.p2p.kademlia.message.OutStreamable;
import at.tugraz.iaik.p2p.kademlia.net.Connector;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.Serializable;

/**
 * A Node in the Kademlia network - Contains basic node network information.
 *
 * @author Joshua Kissoon
 * @version 0.1
 * @since 20140202
 */
public class Node implements OutStreamable, Serializable {

    private final Identifier nodeId;
    private Connector  connector;
    private final String     strRep;
    private final short      nonceRange;

    public Node(final Identifier nid, final short nonceRange, final Connector connector) {
        this.nodeId = nid;
        this.nonceRange = nonceRange;
        this.connector = connector;
        this.strRep = this.nodeId.toString();
    }

    /**
     * Load the Node's data from a DataInput stream
     *
     * @param in
     * @throws IOException
     */
    public Node(final DataInputStream in) throws IOException {
        this(Identifier.fromStr(in), in.readShort(), Connector.Companion.fromStr(in));
    }


    /**
     * @return The NodeId object of this node
     */
    public Identifier getNodeId() {
        return this.nodeId;
    }

    /**
     * Creates a SocketAddress for this node
     *
     * @return
     */
    public Connector getConnector() {
        return this.connector;
    }

    public void setConnector(Connector connector){
        this.connector=connector;
    }

    @Override
    public void toStream(final DataOutputStream out) throws IOException {
        /* Add the NodeId to the stream */

        this.nodeId.toStream(out);
        /* Add the Node's IP address to the stream */
        out.writeShort(nonceRange);
        out.writeByte(this.connector.getType().getID());
        this.connector.toStream(out);

    }


    @Override
    public boolean equals(final Object o) {
        if (o instanceof Node) {
            final Node n = (Node) o;
            if (n == this) {
                return true;
            }
            return this.getNodeId().equals(n.getNodeId());
        }
        return false;
    }

    @Override
    public int hashCode() {
        return this.getNodeId().hashCode();
    }

    @Override
    public String toString() {
        return strRep;
    }

    public short getNonceRange() {
        return nonceRange;
    }
}
