package at.tugraz.iaik.p2p.kademlia.simulations;

import at.tugraz.iaik.p2p.kademlia.JKademliaNode;
import at.tugraz.iaik.p2p.kademlia.node.KademliaID;

import java.io.IOException;
import java.util.Scanner;

/**
 * Testing how the routing table works and it's state after different operations
 *
 * @author Joshua Kissoon
 * @since 20140426
 */
public class RoutingTableStateTesting {

    JKademliaNode[] kads;

    public int numKads = 10;

    public RoutingTableStateTesting() {
        try {
            /* Setting up Kad networks */
            this.kads = new JKademliaNode[this.numKads];

            this.kads[0] = new JKademliaNode("user0", new KademliaID("HRF456789SD584567460"), 1334);
            this.kads[1] = new JKademliaNode("user1", new KademliaID("ASF456789475DS567461"), 1209);
            this.kads[2] = new JKademliaNode("user2", new KademliaID("AFG45678947584567462"), 4585);
            this.kads[3] = new JKademliaNode("user3", new KademliaID("FSF45J38947584567463"), 8104);
            this.kads[4] = new JKademliaNode("user4", new KademliaID("ASF45678947584567464"), 8335);
            this.kads[5] = new JKademliaNode("user5", new KademliaID("GHF4567894DR84567465"), 13345);
            this.kads[6] = new JKademliaNode("user6", new KademliaID("ASF45678947584567466"), 12049);
            this.kads[7] = new JKademliaNode("user7", new KademliaID("AE345678947584567467"), 14585);
            this.kads[8] = new JKademliaNode("user8", new KademliaID("ASAA5678947584567468"), 18104);
            this.kads[9] = new JKademliaNode("user9", new KademliaID("ASF456789475845674U9"), 18335);

            for (int i = 1; i < this.numKads; i++) {
                this.kads[i].bootstrap(this.kads[0].getNode());
            }

            /* Lets shut down a node and then try putting a content on the network. We'll then see how the
            un-responsive contacts work */
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

//    public KadContent putContent(final String content, final JKademliaNode owner) {
//        DHTContentImpl c = null;
////        try {
//        c = new DHTContentImpl(owner.getOwnerId(), "Some Data", owner.getMessage().getNodeId().getType());
////             owner.put(c);
//        return c;
////        } catch (final IOException e) {
////            System.err.println("Error whiles putting content " + content + " from owner: " + owner.getOwnerId());
////        }
//
//        return c;
//    }

    public void shutdownKad(final JKademliaNode kad) {
        try {
            kad.shutdown(false);
        } catch (final IOException ex) {
            System.err.println("Error whiles shutting down node with owner: " + kad.getOwnerId());
        }
    }

    public void printRoutingTable(final int kadId) {
        System.out.println(this.kads[kadId].getRoutingTable());
    }

    public void printRoutingTables() {
        for (int i = 0; i < this.numKads; i++) {
            this.printRoutingTable(i);
        }
    }

    public void printStorage(final int kadId) {
        System.out.println(this.kads[kadId].getDHT());
    }

    public void printStorage() {
        for (int i = 0; i < this.numKads; i++) {
            this.printStorage(i);
        }
    }

    public static void main(final String[] args) {

        final RoutingTableStateTesting rtss = new RoutingTableStateTesting();

        try {
            rtss.printRoutingTables();

            /* Lets shut down a node to test the node removal operation */
            rtss.shutdownKad(rtss.kads[3]);

            // rtss.putContent("Content owned by kad0", rtss.kads[0]);
            rtss.printStorage();

            Thread.sleep(1000);

            /* kad3 should be removed from their routing tables by now. */
            rtss.printRoutingTables();
        } catch (final InterruptedException ex) {

        }

        final Scanner sc = new Scanner(System.in);
        while (true) {
            System.out.println("\n\n ************************* Options **************************** \n");
            System.out.println("1 i - Print routing table of node i");
            final int val1 = sc.nextInt();
            final int val2 = sc.nextInt();

            switch (val1) {
                case 1:
                    rtss.printRoutingTable(val2);
                    break;
            }
        }
    }
}
