package at.tugraz.iaik.p2p.kademlia.net

import at.tugraz.iaik.p2p.kademlia.KadStatistician
import at.tugraz.iaik.p2p.kademlia.message.Message
import at.tugraz.iaik.p2p.kademlia.message.OutStreamable
import mu.KLogger
import mu.KotlinLogging
import java.io.Closeable
import java.io.DataInputStream
import java.io.IOException
import java.net.InetAddress

internal val logger = KotlinLogging.logger {}

/**
 * Created by Bernd Prünster <bernd.pruenster></bernd.pruenster>@iaik.tugraz.at>
 */
abstract class Connector(val type: Type, val statistician: KadStatistician?) : OutStreamable, Closeable {


    @Throws(IOException::class)
    abstract fun receive(): MessageMapping

    @Throws(IOException::class)
    abstract fun send(msg: Message)

    @Throws(IOException::class)
    abstract fun open()

    abstract val destinationKnown:Boolean

    enum class Type constructor(val iD: Byte) {
        UDP(1.toByte()), ECUDP(2.toByte());


        companion object {
            @Throws(IOException::class)
            fun fromID(id: Byte): Type {
                Type.values()
                        .filter { it.iD == id }
                        .forEach { return it }
                throw IOException("cannot find connector of type " + id)
            }
        }
    }

    companion object {
        @Throws(IOException::class)
        fun fromStr(`in`: DataInputStream): Connector {
            val type = Connector.Type.fromID(`in`.readByte())
            val addr = readAddr(`in`)
            val port = `in`.readInt()
            return when (type) {
                Connector.Type.UDP -> UdpConnector(addr, port)
                Connector.Type.ECUDP -> EcUdpConnector(signingKey = null, address = addr, port = port)
            }
        }

        private fun readAddr(str: DataInputStream): InetAddress? {
            val addr = ByteArray(str.readByte().toInt())
            if(addr.isEmpty())
                return null
            str.read(addr)
            return InetAddress.getByAddress(addr)
        }
    }
}

data class MessageMapping(val id: Int, val msg: Message)

