package at.tugraz.iaik.p2p.kademlia.net

import at.tugraz.iaik.p2p.commons.CommonCrypto
import at.tugraz.iaik.p2p.commons.CryptoException
import at.tugraz.iaik.p2p.kademlia.KadStatistician
import at.tugraz.iaik.p2p.kademlia.message.Message
import at.tugraz.iaik.p2p.kademlia.message.MessageFactory
import at.tugraz.iaik.p2p.kademlia.node.KeyBasedID
import java.io.ByteArrayInputStream
import java.io.DataInputStream
import java.io.IOException
import java.net.DatagramPacket
import java.net.InetAddress
import java.security.interfaces.ECPrivateKey

/**
 * Created by Bernd Prünster <bernd.pruenster@iaik.tugraz.at>
 */

class EcUdpConnector @JvmOverloads constructor(private val signingKey: ECPrivateKey?,
                                               address: InetAddress? = InetAddress.getLoopbackAddress(),
                                               port: Int,
                                               statistician: KadStatistician? = null) : UdpConnector(Connector.Type.ECUDP,
                                                                                                     address,
                                                                                                     port,
                                                                                                     statistician) {

    @Throws(IOException::class) override fun serialiseMessage(msg: Message): ByteArray {
        if (signingKey == null) {
            throw IOException("signing key nit set!")
        }
        val data = super.serialiseMessage(msg)
        if (msg.origin.connector is EcUdpConnector) {
            try {
                return CommonCrypto.sign(signingKey, data)
            } catch (e: CryptoException) {
                throw IOException(e)
            }

        }
        throw IOException("Can only send to EcUdpConnector")
    }

    @Throws(IOException::class) override fun parsePacket(packet: DatagramPacket): ByteArray {


        try {
            //TODO udp packet offset
            val data = super.parsePacket(packet)
            val bIn = ByteArrayInputStream(data)
            val `in` = DataInputStream(bIn)
            `in`.skipBytes(4)
            val message = MessageFactory.createMessage(`in`)
            `in`.close()
            //  if (!(message.getOrigin().getConnector() instanceof EcUdpConnector)) return data;
            val id = message.origin.nodeId as KeyBasedID
            //if ((message !is NodeLookupMessage) and (message !is NodeReplyMessage))
            logger.debug { "validating signature for $message" }
            return CommonCrypto.validate(data, id.key!!)

        } catch (e: CryptoException) {
            throw IOException(e)
        }

    }


}
