package at.tugraz.iaik.p2p.kademlia.simulations;

import at.tugraz.iaik.p2p.commons.CommonCrypto;
import at.tugraz.iaik.p2p.commons.CryptoException;
import at.tugraz.iaik.p2p.commons.ECKeyPair;
import at.tugraz.iaik.p2p.kademlia.JKademliaNode;
import at.tugraz.iaik.p2p.kademlia.net.EcUdpConnector;
import at.tugraz.iaik.p2p.kademlia.node.Identifier;
import at.tugraz.iaik.p2p.kademlia.node.KeyBasedID;
import at.tugraz.iaik.p2p.kademlia.node.Node;
import at.tugraz.iaik.p2p.kademlia.node.SHA3ID;
import at.tugraz.iaik.p2p.kademlia.routing.BackRef;
import at.tugraz.iaik.p2p.kademlia.routing.BackRefComparator;
import at.tugraz.iaik.p2p.kademlia.routing.BackRefs;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.TreeSet;

/**
 * Test 1: Try sending a simple message between nodes
 *
 * @author Joshua Kissoon
 * @created 20140218
 */
public class StorageTest {
    @Test
    public void main() throws CryptoException

    {
        System.out.println("Testing distribution of Data based on virtualised node IDs");
        JKademliaNode kad1 = null;
        JKademliaNode kad2 = null;
        JKademliaNode kad3 = null;
        try {
            final ECKeyPair kp1 = CommonCrypto.genKP();
            final ECKeyPair kp2 = CommonCrypto.genKP();
            final ECKeyPair kp3 = CommonCrypto.genKP();

            //            final JKademliaNode kad1 = new JKademliaNode("Joshua", new KeyBasedID((ECPublicKey) kp1.getPublic()), new
            //                    UdpConnector(7571));
            //            final JKademliaNode kad2 = new JKademliaNode("Crystal", new KeyBasedID((ECPublicKey) kp2.getPublic()),
            //                    new UdpConnector(7572));


            kad1 = new JKademliaNode("Joshua",
                    new KeyBasedID(kp1.getPublic()),
                    (short) 100,
                    new EcUdpConnector(kp1.getPrivate(), 7571));
            kad2 = new JKademliaNode("Crystal",
                    new KeyBasedID(kp2.getPublic()),
                    (short) 300,
                    new EcUdpConnector(kp2.getPrivate(), 7572));
            kad3 = new JKademliaNode("Crystal",
                    new KeyBasedID(kp3.getPublic()),
                    (short) 10,
                    new EcUdpConnector(kp3.getPrivate(), 7573));
            kad1.bootstrap(kad2.getNode());
            kad1.bootstrap(kad3.getNode());
            System.out.println(kad1.getRoutingTable());


            Map<Node, Integer> stats = new HashMap<>();
            System.out.println("\n\nGetting closest nodes for 1000 different data hashes");
            for (int i = 0; i < 1000; ++i) {

                SHA3ID sha3ID = new SHA3ID(("foo" + i).getBytes());
                List<Node> nodesForData = kad3.getNodesForData(sha3ID);
                //      System.out.println("\nSHA3ID for data = " + sha3ID + "\nClosest Node: ");
                int index = 0;
                for (Node n : nodesForData) {
                    if (index == 0) {
                        if (stats.containsKey(n)) {
                            stats.put(n, stats.get(n) + 1);
                        } else {
                            stats.put(n, 1);
                        }
                    }

                    TreeSet<BackRef> backRefs = new TreeSet<>(new BackRefComparator(sha3ID.getInt()));
                    backRefs.addAll(BackRefs.toBackRef(n));
                    //        System.out.println(backRefs.first());
                    index++;
                }
            }

            System.out.println(
                    "\n\nStatistics:\n=========================     Node     =========================| Virtual IDs | Times Selected as Closest");
            for (Map.Entry<Node, Integer> e : stats.entrySet()) {
                System.out.println(e.getKey().getNodeId() + "      " + e.getKey().getNonceRange() +"\t\t" + e.getValue());
            }
        } catch (final IOException e) {

        }
        try {
            kad1.shutdown(false);
            kad2.shutdown(false);
        } catch (final IOException e) {

        }
    }
}
